import requests, typing, abc

from ..credentials import resolve_headers
from ..utils.environment import CONNECTORS_EXECUTE_URL, LOCAL_CONNECTORS_EXECUTE_URL


class ConnectorsApiHttpClient(abc.ABC):
    execute_url: str

    def __init__(self, execute_url: str) -> None:
        self.execute_url = execute_url

    def execute(self, name: str, method: str, params: typing.Dict) -> requests.Response:
        raise NotImplementedError()


class ProductionConnectorsApiHttpClient(ConnectorsApiHttpClient):
    def execute(self, name: str, method: str, params: typing.Dict) -> requests.Response:
        body = {"name": name, "method": method, "params": params}
        return requests.post(self.execute_url, json=body)


class LocalConnectorsApiHttpClient(ConnectorsApiHttpClient):
    def execute(self, name: str, method: str, params: typing.Dict) -> requests.Response:
        body = {"name": name, "method": method, "params": params}
        headers = resolve_headers()
        if headers is None:
            raise Exception("You must be logged in to execute a connector method")
        return requests.post(self.execute_url, headers=headers, json=body)


def connectors_api_http_client_factory() -> ConnectorsApiHttpClient:
    if CONNECTORS_EXECUTE_URL is None:
        return LocalConnectorsApiHttpClient(LOCAL_CONNECTORS_EXECUTE_URL)
    else:
        return ProductionConnectorsApiHttpClient(CONNECTORS_EXECUTE_URL)



import abstra.workflows as aw

"""
Abstra scripts are the simplest way to run code in your workflows.
"""


def sum(a, b):
    return a, b


# You can save and get information from the workflow context
stage = aw.get_stage()

stage["result"] = sum(2, 3)

print(f"Script ran!")



import pathlib
from .settings import Settings
from .utils.environment import FILES_FOLDER


def get_persistent_dir() -> pathlib.Path:
    if FILES_FOLDER:
        return pathlib.Path(FILES_FOLDER)

    dir = Settings.root_path / ".abstra" / "persistent"
    dir.mkdir(parents=True, exist_ok=True)
    return dir



from abstra_internals.execution.execution import get_execution_throwable
from abstra_internals.repositories.stage_run import StageRun
from typing import List, Optional
from abstra_internals.repositories import StageRunRepository


def get_stage() -> StageRun:
    execution = get_execution_throwable()
    if execution.stage_run is None:
        raise Exception("Stage run not set")
    return execution.stage_run


def set_stage(**kwargs) -> Optional[StageRun]:
    execution = get_execution_throwable()
    execution.set_stage(**kwargs)
    return execution.stage_run


def next_stage(next_stage_runs: Optional[List[dict]] = None):
    """
    :param next_stage_runs: List of dicts with the following structure:
        {
            "assignee": "user@email.com",
            "data": {
                "key": "value"
            },
            "stage": "stage_name"
        }
    """
    execution = get_execution_throwable()

    execution.advance_stage(next_stage_runs, False)



import typing, threading
from abstra_internals.contract import dashes_contract
from abstra_internals.execution.live_execution import (
    LiveExecution,
    get_live_execution_throwable,
)
from .common import get_query_params, get_user, redirect, alert, execute_js  # exported


def __set_refresh_interval(
    execution: LiveExecution,
    interval: int,
    user_callback: typing.Optional[typing.Callable],
):
    def timer_callback():
        if user_callback:
            user_callback()
        if execution.connected:
            execution.send(dashes_contract.SetupRefreshIntervalMessage())
            __set_refresh_interval(execution, interval, user_callback)

    timer = threading.Timer(interval, timer_callback)
    execution.context["refresh_interval"] = timer
    timer.start()


def start_refresh_interval(
    interval: int, user_callback: typing.Optional[typing.Callable] = None
):
    """Experimental"""
    execution = get_live_execution_throwable()
    __set_refresh_interval(execution, interval, user_callback)


def stop_refresh_interval():
    """Experimental"""
    execution = get_live_execution_throwable()
    timer = execution.context.get("refresh_interval")
    if timer:
        timer.cancel()
        execution.context["refresh_interval"] = None



import datetime, time, re
from ..widget_base import Input
from typing import Union, Optional


class DateInput(Input):
    type = "date-input"
    empty_value = None

    def __init__(self, key: str, label: str, **kwargs):
        super().__init__(key)
        self.set_props(dict(label=label, **kwargs))

    def set_props(self, props):
        self.label = props.get("label", "Label")
        self.value = props.get("initial_value", self.empty_value)
        self.required = props.get("required", True)
        self.hint = props.get("hint", None)
        self.full_width = props.get("full_width", False)
        self.disabled = props.get("disabled", False)

    def render(self, context: dict):
        return {
            "type": self.type,
            "key": self.key,
            "hint": self.hint,
            "label": self.label,
            "value": self.serialize_value(),
            "required": self.required,
            "fullWidth": self.full_width,
            "disabled": self.disabled,
            "errors": self.errors,
        }

    def serialize_value(self) -> str:
        if isinstance(self.value, datetime.date):
            return self.value.isoformat()
        if isinstance(self.value, time.struct_time):
            return (
                datetime.datetime.fromtimestamp(time.mktime(self.value))
                .date()
                .isoformat()
            )
        if isinstance(self.value, str) and re.match(
            "^\\d{4}-\\d{2}-\\d{2}$", self.value
        ):
            return self.value
        return ""

    def parse_value(self, value: str) -> Optional[datetime.date]:
        if not value:
            return None
        try:
            split_value = value.split("T")[0].split("-")
            year = int(split_value[0])
            month = int(split_value[1])
            day = int(split_value[2])
            return datetime.date(year, month, day)
        except Exception:
            raise ValueError("Invalid date string format. Expected YYYY-MM-DD.")



import abstra.dashes as ad

"""
Abstra dashes are the simplest way to build custom user interfaces for your APIs.
"""

x = 2
y = 3


def get_sum():
    return x + y


def show_sum():
    ad.alert(str(get_sum()))
